@@ -4,9 +4,10 @@ import io
 import pathlib
 import sys
 import urllib.parse
+import warnings
 from collections.abc import Callable, Iterable
 from http.cookies import BaseCookie, Morsel, SimpleCookie
-from typing import Any, Optional, Protocol, Union
+from typing import Any, AsyncIterator, Optional, Protocol, Union
 from unittest import mock
 
 import pytest
@@ -204,7 +205,7 @@ def test_host_port_nondefault_wss(make_request) -> None:
 
 def test_host_port_none_port(make_request) -> None:
     req = make_request("get", "unix://localhost/path")
-    assert req.headers["Host"] == "localhost"
+    assert req.headers[hdrs.HOST] == "localhost"
 
 
 def test_host_port_err(make_request) -> None:
@@ -219,17 +220,17 @@ def test_hostname_err(make_request) -> None:
 
 def test_host_header_host_first(make_request) -> None:
     req = make_request("get", "http://python.org/")
-    assert list(req.headers)[0] == "Host"
+    assert list(req.headers)[0] == hdrs.HOST
 
 
 def test_host_header_host_without_port(make_request) -> None:
     req = make_request("get", "http://python.org/")
-    assert req.headers["HOST"] == "python.org"
+    assert req.headers[hdrs.HOST] == "python.org"
 
 
 def test_host_header_host_with_default_port(make_request) -> None:
     req = make_request("get", "http://python.org:80/")
-    assert req.headers["HOST"] == "python.org"
+    assert req.headers[hdrs.HOST] == "python.org"
 
 
 def test_host_header_host_with_nondefault_port(make_request) -> None:
@@ -347,12 +348,12 @@ def test_skip_default_useragent_header(make_request) -> None:
 
 def test_headers(make_request) -> None:
     req = make_request(
-        "post", "http://python.org/", headers={"Content-Type": "text/plain"}
+        "post", "http://python.org/", headers={hdrs.CONTENT_TYPE: "text/plain"}
     )
 
-    assert "CONTENT-TYPE" in req.headers
-    assert req.headers["CONTENT-TYPE"] == "text/plain"
-    assert req.headers["ACCEPT-ENCODING"] == "gzip, deflate, br"
+    assert hdrs.CONTENT_TYPE in req.headers
+    assert req.headers[hdrs.CONTENT_TYPE] == "text/plain"
+    assert req.headers[hdrs.ACCEPT_ENCODING] == "gzip, deflate, br"
 
 
 def test_headers_list(make_request) -> None:
@@ -750,6 +751,7 @@ async def test_post_data(loop: asyncio.AbstractEventLoop, conn: mock.Mock) -> No
         )
         resp = await req.send(conn)
         assert "/" == req.url.path
+        assert isinstance(req.body, payload.Payload)
         assert b"life=42" == req.body._value
         assert "application/x-www-form-urlencoded" == req.headers["CONTENT-TYPE"]
         await req.close()
@@ -787,6 +789,7 @@ async def test_get_with_data(loop) -> None:
             meth, URL("http://python.org/"), data={"life": "42"}, loop=loop
         )
         assert "/" == req.url.path
+        assert isinstance(req.body, payload.Payload)
         assert b"life=42" == req.body._value
         await req.close()
 
@@ -893,6 +896,7 @@ async def test_chunked_explicit(loop, conn) -> None:
     req = ClientRequest("post", URL("http://python.org/"), chunked=True, loop=loop)
     with mock.patch("aiohttp.client_reqrep.StreamWriter") as m_writer:
         m_writer.return_value.write_headers = mock.AsyncMock()
+        m_writer.return_value.write_eof = mock.AsyncMock()
         resp = await req.send(conn)
 
     assert "chunked" == req.headers["TRANSFER-ENCODING"]
@@ -949,7 +953,70 @@ async def test_precompressed_data_stays_intact(loop) -> None:
     await req.close()
 
 
-async def test_file_upload_not_chunked_seek(loop) -> None:
+async def test_body_with_size_sets_content_length(
+    loop: asyncio.AbstractEventLoop,
+) -> None:
+    """Test that when body has a size and no Content-Length header is set, it gets added."""
+    # Create a BytesPayload which has a size property
+    data = b"test data"
+
+    # Create request with data that will create a BytesPayload
+    req = ClientRequest(
+        "post",
+        URL("http://python.org/"),
+        data=data,
+        loop=loop,
+    )
+
+    # Verify Content-Length was set from body.size
+    assert req.headers["CONTENT-LENGTH"] == str(len(data))
+    assert req.body is not None
+    assert req._body is not None  # When _body is set, body returns it
+    assert req._body.size == len(data)
+    await req.close()
+
+
+async def test_body_payload_with_size_no_content_length(
+    loop: asyncio.AbstractEventLoop,
+) -> None:
+    """Test that when a body payload is set via update_body, Content-Length is added."""
+    # Create a payload with a known size
+    data = b"payload data"
+    bytes_payload = payload.BytesPayload(data)
+
+    # Create request with no data initially
+    req = ClientRequest(
+        "post",
+        URL("http://python.org/"),
+        loop=loop,
+    )
+
+    # Initially no body should be set
+    assert req._body is None
+    # POST method with None body should have Content-Length: 0
+    assert req.headers[hdrs.CONTENT_LENGTH] == "0"
+
+    # Update body using the public method
+    await req.update_body(bytes_payload)
+
+    # Verify Content-Length was set from body.size
+    assert req.headers[hdrs.CONTENT_LENGTH] == str(len(data))
+    assert req.body is bytes_payload
+    assert req._body is bytes_payload  # Access _body which is the Payload
+    assert req._body is not None  # type: ignore[unreachable]
+    assert req._body.size == len(data)
+
+    # Set body back to None
+    await req.update_body(None)
+
+    # Verify Content-Length is back to 0 for POST with None body
+    assert req.headers[hdrs.CONTENT_LENGTH] == "0"
+    assert req._body is None
+
+    await req.close()
+
+
+async def test_file_upload_not_chunked_seek(loop: asyncio.AbstractEventLoop) -> None:
     file_path = pathlib.Path(__file__).parent / "aiohttp.png"
     with file_path.open("rb") as f:
         f.seek(100)
@@ -1201,6 +1268,7 @@ async def test_custom_response_class(loop, conn) -> None:
 
 async def test_oserror_on_write_bytes(loop, conn) -> None:
     req = ClientRequest("POST", URL("http://python.org/"), loop=loop)
+    req.body = b"test data"  # type: ignore[assignment]  # https://github.com/python/mypy/issues/12892
 
     writer = WriterMock()
     writer.write.side_effect = OSError
@@ -1557,7 +1625,7 @@ async def test_write_bytes_with_content_length_limit(
     data = b"Hello World"
     req = ClientRequest("post", URL("http://python.org/"), loop=loop)
 
-    req.body = data
+    req.body = data  # type: ignore[assignment]  # https://github.com/python/mypy/issues/12892
 
     writer = StreamWriter(protocol=conn.protocol, loop=loop)
     # Use content_length=5 to truncate data
@@ -1584,7 +1652,17 @@ async def test_write_bytes_with_iterable_content_length_limit(
     """Test that write_bytes respects content_length limit for iterable data."""
     # Test with iterable data
     req = ClientRequest("post", URL("http://python.org/"), loop=loop)
-    req.body = data
+
+    # Convert list to async generator if needed
+    if isinstance(data, list):
+
+        async def gen() -> AsyncIterator[bytes]:
+            for chunk in data:
+                yield chunk
+
+        req.body = gen()  # type: ignore[assignment]  # https://github.com/python/mypy/issues/12892
+    else:
+        req.body = data  # type: ignore[assignment]  # https://github.com/python/mypy/issues/12892
 
     writer = StreamWriter(protocol=conn.protocol, loop=loop)
     # Use content_length=7 to truncate at the middle of Part2
@@ -1599,7 +1677,13 @@ async def test_write_bytes_empty_iterable_with_content_length(
 ) -> None:
     """Test that write_bytes handles empty iterable body with content_length."""
     req = ClientRequest("post", URL("http://python.org/"), loop=loop)
-    req.body = []  # Empty iterable
+
+    # Create an empty async generator
+    async def gen() -> AsyncIterator[bytes]:
+        return
+        yield  # pragma: no cover  # This makes it a generator but never executes
+
+    req.body = gen()  # type: ignore[assignment]  # https://github.com/python/mypy/issues/12892
 
     writer = StreamWriter(protocol=conn.protocol, loop=loop)
     # Use content_length=10 with empty body
@@ -1608,3 +1692,537 @@ async def test_write_bytes_empty_iterable_with_content_length(
     # Verify nothing was written
     assert len(buf) == 0
     await req.close()
+
+
+async def test_warn_if_unclosed_payload_via_body_setter(
+    make_request: _RequestMaker,
+) -> None:
+    """Test that _warn_if_unclosed_payload is called when setting body with unclosed payload."""
+    req = make_request("POST", "http://python.org/")
+
+    # First set a payload that needs manual closing (autoclose=False)
+    file_payload = payload.BufferedReaderPayload(
+        io.BufferedReader(io.BytesIO(b"test data")),  # type: ignore[arg-type]
+        encoding="utf-8",
+    )
+    req.body = file_payload
+
+    # Setting body again should trigger the warning for the previous payload
+    with pytest.warns(
+        ResourceWarning,
+        match="The previous request body contains unclosed resources",
+    ):
+        req.body = b"new data"  # type: ignore[assignment]  # https://github.com/python/mypy/issues/12892
+
+    await req.close()
+
+
+async def test_no_warn_for_autoclose_payload_via_body_setter(
+    make_request: _RequestMaker,
+) -> None:
+    """Test that no warning is issued for payloads with autoclose=True."""
+    req = make_request("POST", "http://python.org/")
+
+    # First set BytesIOPayload which has autoclose=True
+    bytes_payload = payload.BytesIOPayload(io.BytesIO(b"test data"))
+    req.body = bytes_payload
+
+    # Setting body again should not trigger warning since previous payload has autoclose=True
+    with warnings.catch_warnings(record=True) as warning_list:
+        warnings.simplefilter("always")
+        req.body = b"new data"  # type: ignore[assignment]  # https://github.com/python/mypy/issues/12892
+
+    # Filter out any non-ResourceWarning warnings
+    resource_warnings = [
+        w for w in warning_list if issubclass(w.category, ResourceWarning)
+    ]
+    assert len(resource_warnings) == 0
+
+    await req.close()
+
+
+async def test_no_warn_for_consumed_payload_via_body_setter(
+    make_request: _RequestMaker,
+) -> None:
+    """Test that no warning is issued for already consumed payloads."""
+    req = make_request("POST", "http://python.org/")
+
+    # Create a payload that needs manual closing
+    file_payload = payload.BufferedReaderPayload(
+        io.BufferedReader(io.BytesIO(b"test data")),  # type: ignore[arg-type]
+        encoding="utf-8",
+    )
+    req.body = file_payload
+
+    # Properly close the payload to mark it as consumed
+    await file_payload.close()
+
+    # Setting body again should not trigger warning since previous payload is consumed
+    with warnings.catch_warnings(record=True) as warning_list:
+        warnings.simplefilter("always")
+        req.body = b"new data"  # type: ignore[assignment]  # https://github.com/python/mypy/issues/12892
+
+    # Filter out any non-ResourceWarning warnings
+    resource_warnings = [
+        w for w in warning_list if issubclass(w.category, ResourceWarning)
+    ]
+    assert len(resource_warnings) == 0
+
+    await req.close()
+
+
+async def test_warn_if_unclosed_payload_via_update_body_from_data(
+    make_request: _RequestMaker,
+) -> None:
+    """Test that _warn_if_unclosed_payload is called via update_body_from_data."""
+    req = make_request("POST", "http://python.org/")
+
+    # First set a payload that needs manual closing
+    file_payload = payload.BufferedReaderPayload(
+        io.BufferedReader(io.BytesIO(b"initial data")),  # type: ignore[arg-type]
+        encoding="utf-8",
+    )
+    req.update_body_from_data(file_payload)
+
+    # Create FormData for second update
+    form = aiohttp.FormData()
+    form.add_field("test", "value")
+
+    # update_body_from_data should trigger the warning for the previous payload
+    with pytest.warns(
+        ResourceWarning,
+        match="The previous request body contains unclosed resources",
+    ):
+        req.update_body_from_data(form)
+
+    await req.close()
+
+
+async def test_warn_via_update_with_file_payload(
+    make_request: _RequestMaker,
+) -> None:
+    """Test warning via update_body_from_data with file-like object."""
+    req = make_request("POST", "http://python.org/")
+
+    # First create a file-like object that results in BufferedReaderPayload
+    buffered1 = io.BufferedReader(io.BytesIO(b"file content 1"))  # type: ignore[arg-type]
+    req.update_body_from_data(buffered1)
+
+    # Second update should warn about the first payload
+    buffered2 = io.BufferedReader(io.BytesIO(b"file content 2"))  # type: ignore[arg-type]
+
+    with pytest.warns(
+        ResourceWarning,
+        match="The previous request body contains unclosed resources",
+    ):
+        req.update_body_from_data(buffered2)
+
+    await req.close()
+
+
+async def test_no_warn_for_simple_data_via_update_body_from_data(
+    make_request: _RequestMaker,
+) -> None:
+    """Test that no warning is issued for simple data types."""
+    req = make_request("POST", "http://python.org/")
+
+    # Simple bytes data should not trigger warning
+    with warnings.catch_warnings(record=True) as warning_list:
+        warnings.simplefilter("always")
+        req.update_body_from_data(b"simple data")
+
+    # Filter out any non-ResourceWarning warnings
+    resource_warnings = [
+        w for w in warning_list if issubclass(w.category, ResourceWarning)
+    ]
+    assert len(resource_warnings) == 0
+
+    await req.close()
+
+
+async def test_update_body_closes_previous_payload(
+    make_request: _RequestMaker,
+) -> None:
+    """Test that update_body properly closes the previous payload."""
+    req = make_request("POST", "http://python.org/")
+
+    # Create a mock payload that tracks if it was closed
+    mock_payload = mock.Mock(spec=payload.Payload)
+    mock_payload.close = mock.AsyncMock()
+
+    # Set initial payload
+    req._body = mock_payload
+
+    # Update body with new data
+    await req.update_body(b"new body data")
+
+    # Verify the previous payload was closed
+    mock_payload.close.assert_called_once()
+
+    # Verify new body is set (it's a BytesPayload now)
+    assert isinstance(req.body, payload.BytesPayload)
+
+    await req.close()
+
+
+async def test_body_setter_closes_previous_payload(
+    make_request: _RequestMaker,
+) -> None:
+    """Test that body setter properly closes the previous payload."""
+    req = make_request("POST", "http://python.org/")
+
+    # Create a mock payload that tracks if it was closed
+    # We need to use create_autospec to ensure all methods are available
+    mock_payload = mock.create_autospec(payload.Payload, instance=True)
+
+    # Set initial payload
+    req._body = mock_payload
+
+    # Update body with new data using setter
+    req.body = b"new body data"  # type: ignore[assignment]  # https://github.com/python/mypy/issues/12892
+
+    # Verify the previous payload was closed using _close
+    mock_payload._close.assert_called_once()
+
+    # Verify new body is set (it's a BytesPayload now)
+    assert isinstance(req.body, payload.BytesPayload)
+
+    await req.close()
+
+
+async def test_update_body_with_different_types(
+    make_request: _RequestMaker,
+) -> None:
+    """Test update_body with various data types."""
+    req = make_request("POST", "http://python.org/")
+
+    # Test with bytes
+    await req.update_body(b"bytes data")
+    assert isinstance(req.body, payload.BytesPayload)
+
+    # Test with string
+    await req.update_body("string data")
+    assert isinstance(req.body, payload.BytesPayload)
+
+    # Test with None (clears body)
+    await req.update_body(None)
+    assert req.body == b""  # type: ignore[comparison-overlap]  # empty body is represented as b""
+
+    await req.close()
+
+
+async def test_update_body_with_chunked_encoding(
+    make_request: _RequestMaker,
+) -> None:
+    """Test that update_body properly handles chunked transfer encoding."""
+    # Create request with chunked=True
+    req = make_request("POST", "http://python.org/", chunked=True)
+
+    # Verify Transfer-Encoding header is set
+    assert req.headers["Transfer-Encoding"] == "chunked"
+    assert "Content-Length" not in req.headers
+
+    # Update body - should maintain chunked encoding
+    await req.update_body(b"chunked data")
+    assert req.headers["Transfer-Encoding"] == "chunked"
+    assert "Content-Length" not in req.headers
+    assert isinstance(req.body, payload.BytesPayload)
+
+    # Update with different body - chunked should remain
+    await req.update_body(b"different chunked data")
+    assert req.headers["Transfer-Encoding"] == "chunked"
+    assert "Content-Length" not in req.headers
+
+    # Clear body - chunked header should remain
+    await req.update_body(None)
+    assert req.headers["Transfer-Encoding"] == "chunked"
+    assert "Content-Length" not in req.headers
+
+    await req.close()
+
+
+async def test_update_body_get_method_with_none_body(
+    make_request: _RequestMaker,
+) -> None:
+    """Test that update_body with GET method and None body doesn't call update_transfer_encoding."""
+    # Create GET request
+    req = make_request("GET", "http://python.org/")
+
+    # GET requests shouldn't have Transfer-Encoding or Content-Length initially
+    assert "Transfer-Encoding" not in req.headers
+    assert "Content-Length" not in req.headers
+
+    # Update body to None - should not trigger update_transfer_encoding
+    # This covers the branch where body is None AND method is in GET_METHODS
+    await req.update_body(None)
+
+    # Headers should remain unchanged
+    assert "Transfer-Encoding" not in req.headers
+    assert "Content-Length" not in req.headers
+
+    await req.close()
+
+
+async def test_update_body_updates_content_length(
+    make_request: _RequestMaker,
+) -> None:
+    """Test that update_body properly updates Content-Length header when body size changes."""
+    req = make_request("POST", "http://python.org/")
+
+    # Set initial body with known size
+    await req.update_body(b"initial data")
+    initial_content_length = req.headers.get("Content-Length")
+    assert initial_content_length == "12"  # len(b"initial data") = 12
+
+    # Update body with different size
+    await req.update_body(b"much longer data than before")
+    new_content_length = req.headers.get("Content-Length")
+    assert new_content_length == "28"  # len(b"much longer data than before") = 28
+
+    # Update body with shorter data
+    await req.update_body(b"short")
+    assert req.headers.get("Content-Length") == "5"  # len(b"short") = 5
+
+    # Clear body
+    await req.update_body(None)
+    # For None body with POST method, Content-Length should be set to 0
+    assert req.headers[hdrs.CONTENT_LENGTH] == "0"
+
+    await req.close()
+
+
+async def test_warn_stacklevel_points_to_user_code(
+    make_request: _RequestMaker,
+) -> None:
+    """Test that the warning stacklevel correctly points to user code."""
+    req = make_request("POST", "http://python.org/")
+
+    # First set a payload that needs manual closing (autoclose=False)
+    file_payload = payload.BufferedReaderPayload(
+        io.BufferedReader(io.BytesIO(b"test data")),  # type: ignore[arg-type]
+        encoding="utf-8",
+    )
+    req.body = file_payload
+
+    # Capture warnings with their details
+    with warnings.catch_warnings(record=True) as warning_list:
+        warnings.simplefilter("always", ResourceWarning)
+        # This line should be reported as the warning source
+        req.body = b"new data"  # type: ignore[assignment]  # https://github.com/python/mypy/issues/12892  # LINE TO BE REPORTED
+
+    # Find the ResourceWarning
+    resource_warnings = [
+        w for w in warning_list if issubclass(w.category, ResourceWarning)
+    ]
+    assert len(resource_warnings) == 1
+
+    warning = resource_warnings[0]
+    # The warning should point to the line where we set req.body, not inside the library
+    # Call chain: user code -> body setter -> _warn_if_unclosed_payload
+    # stacklevel=3 is used in body setter to skip the setter and _warn_if_unclosed_payload
+    assert warning.filename == __file__
+    # The line number should be the line with "req.body = b'new data'"
+    # We can't hardcode the line number, but we can verify it's not pointing
+    # to client_reqrep.py (the library code)
+    assert "client_reqrep.py" not in warning.filename
+
+    await req.close()
+
+
+async def test_warn_stacklevel_update_body_from_data(
+    make_request: _RequestMaker,
+) -> None:
+    """Test that warning stacklevel is correct when called from update_body_from_data."""
+    req = make_request("POST", "http://python.org/")
+
+    # First set a payload that needs manual closing (autoclose=False)
+    file_payload = payload.BufferedReaderPayload(
+        io.BufferedReader(io.BytesIO(b"test data")),  # type: ignore[arg-type]
+        encoding="utf-8",
+    )
+    req.update_body_from_data(file_payload)
+
+    # Capture warnings with their details
+    with warnings.catch_warnings(record=True) as warning_list:
+        warnings.simplefilter("always", ResourceWarning)
+        # This line should be reported as the warning source
+        req.update_body_from_data(b"new data")  # LINE TO BE REPORTED
+
+    # Find the ResourceWarning
+    resource_warnings = [
+        w for w in warning_list if issubclass(w.category, ResourceWarning)
+    ]
+    assert len(resource_warnings) == 1
+
+    warning = resource_warnings[0]
+    # For update_body_from_data, stacklevel=3 points to this test file
+    # Call chain: user code -> update_body_from_data -> _warn_if_unclosed_payload
+    assert warning.filename == __file__
+    assert "client_reqrep.py" not in warning.filename
+
+    await req.close()
+
+
+async def test_expect100_with_body_becomes_none() -> None:
+    """Test that write_bytes handles body becoming None after expect100 handling."""
+    # Create a mock writer and connection
+    mock_writer = mock.AsyncMock()
+    mock_conn = mock.Mock()
+
+    # Create a request
+    req = ClientRequest(
+        "POST", URL("http://test.example.com/"), loop=asyncio.get_event_loop()
+    )
+    req._body = mock.Mock()  # Start with a body
+
+    # Now set body to None to simulate a race condition
+    # where req._body is set to None after expect100 handling
+    req._body = None
+
+    await req.write_bytes(mock_writer, mock_conn, None)
+
+
+@pytest.mark.parametrize(
+    ("method", "data", "expected_content_length"),
+    [
+        # GET methods should not have Content-Length with None body
+        ("GET", None, None),
+        ("HEAD", None, None),
+        ("OPTIONS", None, None),
+        ("TRACE", None, None),
+        # POST methods should have Content-Length: 0 with None body
+        ("POST", None, "0"),
+        ("PUT", None, "0"),
+        ("PATCH", None, "0"),
+        ("DELETE", None, "0"),
+        # Empty bytes should always set Content-Length: 0
+        ("GET", b"", "0"),
+        ("HEAD", b"", "0"),
+        ("POST", b"", "0"),
+        ("PUT", b"", "0"),
+        # Non-empty bytes should set appropriate Content-Length
+        ("GET", b"test", "4"),
+        ("POST", b"test", "4"),
+        ("PUT", b"hello world", "11"),
+        ("PATCH", b"data", "4"),
+        ("DELETE", b"x", "1"),
+    ],
+)
+def test_content_length_for_methods(
+    method: str,
+    data: Optional[bytes],
+    expected_content_length: Optional[str],
+    loop: asyncio.AbstractEventLoop,
+) -> None:
+    """Test that Content-Length header is set correctly for all HTTP methods."""
+    req = ClientRequest(method, URL("http://python.org/"), data=data, loop=loop)
+
+    actual_content_length = req.headers.get(hdrs.CONTENT_LENGTH)
+    assert actual_content_length == expected_content_length
+
+
+@pytest.mark.parametrize("method", ["GET", "HEAD", "OPTIONS", "TRACE"])
+def test_get_methods_classification(method: str) -> None:
+    """Test that GET-like methods are correctly classified."""
+    assert method in ClientRequest.GET_METHODS
+
+
+@pytest.mark.parametrize("method", ["POST", "PUT", "PATCH", "DELETE"])
+def test_non_get_methods_classification(method: str) -> None:
+    """Test that POST-like methods are not in GET_METHODS."""
+    assert method not in ClientRequest.GET_METHODS
+
+
+async def test_content_length_with_string_data(loop: asyncio.AbstractEventLoop) -> None:
+    """Test Content-Length when data is a string."""
+    data = "Hello, World!"
+    req = ClientRequest("POST", URL("http://python.org/"), data=data, loop=loop)
+    # String should be encoded to bytes, default encoding is utf-8
+    assert req.headers[hdrs.CONTENT_LENGTH] == str(len(data.encode("utf-8")))
+    await req.close()
+
+
+async def test_content_length_with_async_iterable(
+    loop: asyncio.AbstractEventLoop,
+) -> None:
+    """Test that async iterables use chunked encoding, not Content-Length."""
+
+    async def data_gen() -> AsyncIterator[bytes]:
+        yield b"chunk1"  # pragma: no cover
+
+    req = ClientRequest("POST", URL("http://python.org/"), data=data_gen(), loop=loop)
+    assert hdrs.CONTENT_LENGTH not in req.headers
+    assert req.chunked
+    assert req.headers[hdrs.TRANSFER_ENCODING] == "chunked"
+    await req.close()
+
+
+async def test_content_length_not_overridden(loop: asyncio.AbstractEventLoop) -> None:
+    """Test that explicitly set Content-Length is not overridden."""
+    req = ClientRequest(
+        "POST",
+        URL("http://python.org/"),
+        data=b"test",
+        headers={hdrs.CONTENT_LENGTH: "100"},
+        loop=loop,
+    )
+    # Should keep the explicitly set value
+    assert req.headers[hdrs.CONTENT_LENGTH] == "100"
+    await req.close()
+
+
+async def test_content_length_with_formdata(loop: asyncio.AbstractEventLoop) -> None:
+    """Test Content-Length with FormData."""
+    form = aiohttp.FormData()
+    form.add_field("field", "value")
+
+    req = ClientRequest("POST", URL("http://python.org/"), data=form, loop=loop)
+    # FormData with known size should set Content-Length
+    assert hdrs.CONTENT_LENGTH in req.headers
+    await req.close()
+
+
+async def test_no_content_length_with_chunked(loop: asyncio.AbstractEventLoop) -> None:
+    """Test that chunked encoding prevents Content-Length header."""
+    req = ClientRequest(
+        "POST",
+        URL("http://python.org/"),
+        data=b"test",
+        chunked=True,
+        loop=loop,
+    )
+    assert hdrs.CONTENT_LENGTH not in req.headers
+    assert req.headers[hdrs.TRANSFER_ENCODING] == "chunked"
+    await req.close()
+
+
+@pytest.mark.parametrize("method", ["POST", "PUT", "PATCH", "DELETE"])
+async def test_update_body_none_sets_content_length_zero(
+    method: str, loop: asyncio.AbstractEventLoop
+) -> None:
+    """Test that updating body to None sets Content-Length: 0 for POST-like methods."""
+    # Create request with initial body
+    req = ClientRequest(method, URL("http://python.org/"), data=b"initial", loop=loop)
+    assert req.headers[hdrs.CONTENT_LENGTH] == "7"
+
+    # Update body to None
+    await req.update_body(None)
+    assert req.headers[hdrs.CONTENT_LENGTH] == "0"
+    assert req._body is None
+    await req.close()
+
+
+@pytest.mark.parametrize("method", ["GET", "HEAD", "OPTIONS", "TRACE"])
+async def test_update_body_none_no_content_length_for_get_methods(
+    method: str, loop: asyncio.AbstractEventLoop
+) -> None:
+    """Test that updating body to None doesn't set Content-Length for GET-like methods."""
+    # Create request with initial body
+    req = ClientRequest(method, URL("http://python.org/"), data=b"initial", loop=loop)
+    assert req.headers[hdrs.CONTENT_LENGTH] == "7"
+
+    # Update body to None
+    await req.update_body(None)
+    assert hdrs.CONTENT_LENGTH not in req.headers
+    assert req._body is None
+    await req.close()
